#  - setup.py - 
# This file is part of mp3togo

# Convert audio files to play on a mp3 player
# Manage program options
#
# (c) Simeon Veldstra 2004, 2006  <reallifesim@gmail.com>
#
# This software is free.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You may redistribute this program under the terms of the
# GNU General Public Licence version 2
# Available in this package or at http://www.fsf.org

# requires python-pyvorbis and python-id3 and lame
# and mpg321

import options
import sys, os

class Fail(options.Fail):
	pass

class Options(options.Options):
	"""Subclass options.Options and fill in application specific
	information."""
	def __init__(self, argv=None, conffile=None, readconf=True):
		options.Options.__init__(self, '~/mp3togo')

		# Options to add:
		# ((name, short option, long option, default value, hook, help text), ...)
		self._set += [
 ('brwarning', 't', 'file-tag', '-PORTABLE-', None, 
 '''A string appended to the name of the file to indicate it has been recoded.'''),
 ('tempdir', 'w', 'work-dir', '/tmp', None, 
 '''The path to store temporary files. This could need several hundred megabytes free.'''),
 ('treedepth', 'd', 'tree-depth', 1, None, 
 '''The number of directory levels to preserve in the output tree. Use 0 to put all output files directly into the target directory. Use 2 to create a directory for the Artist and then the Album (Assuming your music collection is organized in directories by artist then album).'''),
 ('maxunits', 'm', 'max-size', '0', self._maxunits, 
 '''The disk space available to use in bytes. Append 'M' for megabytes, 'G' for gigabytes or 'K' for kilobytes. Use 0 to use all available space on the filesystem.'''),
 ('maxsize', '', '', 0L, None, ''),
 ('force', 'F', 'force', False, None, 
 '''Overwrite files if they already exist.'''),
 ('encoder', 'C', 'encoder', 'lame', None, 
 '''The encoder to use to create the output files. Options are wave, lame or oggenc.'''),
 ('encopts', 'E', 'encoder-options', '', None, 
 '''Compression options for the encoder.'''),
 ('compfactor', 'z', 'compression-factor', 10.0, None, 
 '''If you change the lame options, you must change this factor to match the compression rate of the new options. This is used to estimate completed file size.'''),
 ('help', 'h', 'help', False, None, '''Print this message.'''),
 ('playlist', 'p', 'playlist', '', None, 
 '''The playlist to convert. Playlists can be simple lists of file paths, one per line, or .m3u files or the native XMMS playlist file format.'''),
 ('readxmms', 'x', 'import-xmms', False, None, 
 '''Get playlist from running instance of XMMS. (You must have xmms-shell installed)'''),
 ('playerdir', 'o', 'output-dir', os.curdir, self._playerdir, 
 '''Where to write the output files.'''),
 ('index', '', 'index', False, None, 
 '''Create an index file when in wav output mode.'''),
 ('nonormal', '', 'no-normalize', False, None, 
 '''Don't normalize the wav file before encoding.''')]

		# Options to not save to the config file:
		self._nosave_options += []

		# Binaries to check for:
		self.bin['wave'] = True
		self.bin['lame'] = False
		self.bin['oggenc'] = False
		self.bin['mpg321'] = False
		self.bin['ogg123'] = False
		self.bin['flac'] = False
		self.bin['metaflac'] = False
		self.bin['normalize-audio'] = False
		self.bin['xmms-shell'] = False

		self._help_preamble = """
Synopsis:
 mp3togo [-p playlist] [-o output-dir] [options] [input files]

Description:
 mp3togo is a program for converting audio files of various
 formats into a common format suitable for use on a portable
 mp3 player or an mp3 CD. The files to convert can be
 specified in a playlist file (m3u, pls and plain text are
 supported) or given as arguments to the program. mp3togo
 will go through the list and run mpg321, ogg123, flac and
 lame to decode and reencode the files. The newly encoded
 files will be written to the destination directory. The
 software can retain as much of the subdirectory structure
 as desired.
"""

		# Go ahead and read in the data:
		self.getconf(argv, conffile, readconf)

	def _post_hook(self):
		#Set up default encoder options if not specified:
		if not self['encopts']:
			self.reset_encoder_options()

	def _conffile(self, value):
		self._d['conffile'] = absfile(value)
		
	def _playerdir(self, dir):
		self._d['playerdir'] = absfile(dir)

	def _maxunits(self, value):
		#Calculate raw bytes and set maxsize
		try:
			if value[-1] in ('m', 'M'):
				max = long(value[:-1]) * 1048576L
			elif value[-1] in ('g', 'G'):
				max = long(value[:-1]) * 1073741824L
			elif value[-1] in ('k', 'K'):
				max = long(value[:-1]) * 1024L
			else:
				max = long(value)
		except ValueError:
			raise Fail, "Bad max-size option: %s " % value
		self._d['maxsize'] = max
		self._d['maxunits'] = value

	def _arg_files(self, value):
		l = []
		for f in value:
			f = absfile(f)
			if os.path.exists(f):
				l.append(f)
		self._d['arg_files'] = l

	def reset_encoder_options(self):
		"""Reset encoder options to defaults."""
		if self['encoder'] == 'lame':
			self['encopts'] = '--abr 96'
		elif self['encoder'] == 'oggenc':
			self['encopts'] = '-m 96 -M 225 -b 100'
		elif self['encoder'] == 'wave':
			self['encopts'] = ''


def absfile(name):
	name = name.strip()
	name = os.path.expanduser(name)
	name = os.path.abspath(name)
	return name
